
(in-package "LISP")
(unintern (find-symbol "CONDITION-FUNCTION-NAME" "CONDITIONS") "CONDITIONS")
(export 'kernel::condition-function-name "KERNEL")


(dolist (zot '("CONDITION" "ERROR" "SERIOUS-CONDITION" "SIMPLE-CONDITION"
	       "SIMPLE-ERROR" "SIMPLE-WARNING" "STYLE-WARNING" "WARNING"
	       "SIMPLE-TYPE-ERROR"
	       "SIMPLE-STYLE-WARNING"))
  (let ((warn (find-symbol zot "LISP")))
    (unintern warn "LISP")
    (let ((nwarn (intern zot "LISP")))
      (when (fboundp warn) (setf (fdefinition nwarn) (fdefinition warn)))
      (export nwarn "LISP"))))

(defun output-instance (instance stream)
  (let ((layout (typecase instance
		  (instance (%instance-ref instance 0))
		  (funcallable-instance
		   (%funcallable-instance-layout instance)))))

    (if (typep layout 'layout)
	(let ((class (layout-class layout)))
	  (cond ((typep class 'basic-structure-class)
		 (funcall (or (basic-structure-class-print-function class)
			      #'default-structure-print)
			  instance stream *current-level*))
		((typep class 'conditions::condition-class)
		 (print-unreadable-object (instance stream
						    :identity t
						    :type t)))
		((fboundp 'print-object)
		 (print-object instance stream))
		(t
		 (print-unreadable-object (instance stream :identity t)
		   (write-string "Unprintable Instance" stream)))))
	(print-unreadable-object (instance stream :identity t)
	  (write-string "Unprintable Instance" stream)))))

(in-package "KERNEL")
(export '(slot-class-print-function make-layout make-undefined-class
				    insured-find-class
				    redefine-layout-warning slot-class))

(defun %compiler-only-defstruct (info inherits)
  (multiple-value-bind
      (class layout old-layout)
      (multiple-value-bind
	  (clayout clayout-p)
	  (info type compiler-layout (dd-name info))
	(ensure-structure-class info inherits
				(if clayout-p "previously compiled" "current")
				"compiled"
				clayout))
    (cond
     (old-layout
      (undefine-structure (layout-class old-layout))
      (when (and (class-subclasses class)
		 (not (eq layout old-layout)))
	(collect ((subs))
	  (do-hash (class layout (class-subclasses class))
	    (declare (ignore layout))
	    (undefine-structure class)
	    (subs (class-proper-name class)))
	  (when (subs)
	    (warn "Removing old subclasses of ~S:~%  ~S"
		  (class-name class) (subs))))))
     (t
      (unless (eq (class-layout class) layout)
	(register-layout layout :invalidate nil))
      (setf (find-class (dd-name info)) class)))
    
    (setf (info type compiler-layout (dd-name info)) layout))
  
  (undefined-value))

(defun undefine-structure (class)
  (let ((info (layout-info (class-layout class))))
    (when (defstruct-description-p info)
      (let ((type (dd-name info)))
	(setf (info type compiler-layout type) nil)
	(undefine-function-name (dd-copier info))
	(undefine-function-name (dd-predicate info))
	(dolist (slot (dd-slots info))
	  (let ((fun (dsd-accessor slot)))
	    (undefine-function-name fun)
	    (unless (dsd-read-only slot)
	      (undefine-function-name `(setf ,fun))))))
      ;;
      ;; Clear out the SPECIFIER-TYPE cache so that subsequent references are
      ;; unknown types.
      (values-specifier-type-cache-clear)))
  (undefined-value))

(defstruct (slot-class (:include class))
  ;;
  ;; Print function, or NIL if none.
  (print-function nil :type (or function symbol null)))

(eval-when (compile load eval)
(defun (setf find-class) (new-value name)
  (declare (type class new-value))
  (ecase (info type kind name)
    ((nil))
    (:instance
     (let ((old (class-of (find-class name)))
	   (new (class-of new-value)))
       (unless (eq old new)
	 (warn "Changing meta-class of ~S from ~S to ~S."
	       name (class-name old) (class-name new)))))
    (:primitive
     (error "Illegal to redefine standard type ~S." name))
    (:defined
     (warn "Redefining DEFTYPE type to be a class: ~S."
	   name)
     (setf (info type expander name) nil)))

  (remhash name *forward-referenced-layouts*)
  (%note-type-defined name)
  (setf (info type kind name) :instance)
  (setf (class-cell-class (find-class-cell name)) new-value)
  (unless (eq (info type compiler-layout name)
	      (class-layout new-value))
    (setf (info type compiler-layout name) (class-layout new-value)))
  new-value)

)

(in-package "CONDITIONS")

(eval-when (compile load eval)
(defstruct (condition-class (:include slot-class))
  ;;
  ;; List of CONDITION-SLOT structures for the direct slots of this class.
  (slots nil :type list)
  ;;
  ;; List of CONDITION-SLOT structures for all of the effective class slots of
  ;; this class.
  (class-slots nil :type list)
  ;;
  ;; Report function or NIL.
  (report nil :type (or function null))
  ;;
  ;; List of alternating initargs and initforms.
  (default-initargs () :type list)
  ;;
  ;; CPL as a list of class objects, with all non-condition classes removed.
  (cpl () :type list))

)


(defstruct (condition
	    (:constructor make-condition-object (actual-initargs))
	    (:alternate-metaclass instance condition-class
				  make-condition-class))
		      
  (function-name nil)
  ;;
  ;; Actual initargs supplied to MAKE-CONDITION.
  (actual-initargs (required-argument) :type list)
  ;;
  ;; Plist mapping slot names to any values that were assigned or defaulted
  ;; after creation.
  (assigned-slots () :type list))


(in-package "PCL")
(defun structure-type-p (type)
  (and (symbolp type)
       (let ((class  (lisp:find-class type nil)))
	 (and class)
	 (typep (kernel:layout-info (kernel:class-layout class))
		'kernel:defstruct-description))))

(in-package "USER")

(compile-file "target:code/class" :output-file nil)
(compile-file "target:code/error" :output-file nil)

(in-package "C")
(export '(%more-arg-values))
